package lupenko.airport.service.impl;

import lombok.extern.slf4j.Slf4j;
import lupenko.airport.dao.BookingRepository;
import lupenko.airport.domain.Booking;
import lupenko.airport.domain.Flight;
import lupenko.airport.exception.AircraftCapacityException;
import lupenko.airport.service.BookingService;
import lupenko.airport.web.dto.BookingDto;
import lupenko.airport.service.map.BookingMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.util.Optional;

@Service
@Slf4j
public class BookingServiceImpl implements BookingService {

    private final BookingRepository bookingRepository;
    private final BookingMapper bookingMapper;

    @Autowired
    public BookingServiceImpl(final BookingRepository bookingRepository, final BookingMapper bookingMapper) {
        this.bookingRepository = bookingRepository;
        this.bookingMapper = bookingMapper;
    }

    @Override
    public Page<BookingDto> getBookings(final Pageable pageable) {
        log.info("Finding all bookings {}", pageable);
        return bookingRepository.findAll(pageable).map(bookingMapper::toDto);
    }

    @Override
    public BookingDto createBooking(final BookingDto dto) {
        log.info("Creating booking {}", dto);
        Booking booking = bookingMapper.dtoToEntity(dto);
        checkAircraftCapacityByFlight(booking.getFlight());
        return bookingMapper.toDto(bookingRepository.save(booking));
    }

    private void checkAircraftCapacityByFlight(final Flight flight) {
        Long capacity = flight.getAircraft().getCapacity();
        int bookings = flight.getBookings().size();
        if (capacity == bookings) {
            throw new AircraftCapacityException(capacity, flight.getAircraft().getId());
        }
    }

    @Override
    public void deleteBooking(final Long id) {
        log.info("Deleting booking {}", id);
        bookingRepository.deleteById(id);
    }

    @Override
    public Optional<BookingDto> getBooking(final Long id) {
        log.info("Fetching booking {}", id);
        return bookingRepository.findById(id).map(bookingMapper::toDto);
    }

    @Override
    public Optional<BookingDto> setCheckoutTime(final Long id, final Instant checkoutTime) {
        log.info("Updating booking{}, checkout time by {}", id, checkoutTime);
        return bookingRepository.findById(id)
                .map((booking) -> {
                    booking.setCheckout(checkoutTime);
                    return booking;
                })
                .map(bookingRepository::save)
                .map(bookingMapper::toDto);
    }

}
