package lupenko.airport.service.map;

import lupenko.airport.dao.AircraftRepository;
import lupenko.airport.dao.AirportRepository;
import lupenko.airport.domain.Aircraft;
import lupenko.airport.domain.Airport;
import lupenko.airport.domain.Flight;
import lupenko.airport.web.dto.FlightDto;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.test.util.ReflectionTestUtils;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class FlightMapperTest {

    private static final long DST_AIRPORT_ID = 13L;
    private static final long SRC_AIRPORT_ID = 37L;
    private static final long AIRCRAFT_ID = 22L;
    private static final Airport DST_AIRPORT_MOCK = mock(Airport.class);
    private static final Airport SRC_AIRPORT_MOCK = mock(Airport.class);
    private static final Aircraft AIRCRAFT_MOCK = mock(Aircraft.class);
    private static final Instant DEPARTURE_TIME = Instant.EPOCH.plus(1, ChronoUnit.DAYS);
    private static final Instant ARRIVAL_TIME = Instant.EPOCH.plus(2, ChronoUnit.DAYS);
    private static final Instant CHECKOUT_START = Instant.EPOCH.plus(5, ChronoUnit.HOURS);
    private static final Instant CHECKOUT_END = Instant.EPOCH.plus(10, ChronoUnit.HOURS);
    private static FlightMapperImpl flightMapper = new FlightMapperImpl();

    @BeforeAll
    public static void mockRepositories() {
        when(DST_AIRPORT_MOCK.getId()).thenReturn(DST_AIRPORT_ID);
        when(SRC_AIRPORT_MOCK.getId()).thenReturn(SRC_AIRPORT_ID);
        when(AIRCRAFT_MOCK.getId()).thenReturn(AIRCRAFT_ID);
        AirportRepository airportRepositoryMock = mock(AirportRepository.class);
        when(airportRepositoryMock.findById(DST_AIRPORT_ID)).thenReturn(Optional.of(DST_AIRPORT_MOCK));
        when(airportRepositoryMock.findById(SRC_AIRPORT_ID)).thenReturn(Optional.of(SRC_AIRPORT_MOCK));
        ReflectionTestUtils.setField(flightMapper, "airportRepository", airportRepositoryMock);

        AircraftRepository aircraftRepositoryMock = mock(AircraftRepository.class);
        when(aircraftRepositoryMock.findById(AIRCRAFT_ID)).thenReturn(Optional.of(AIRCRAFT_MOCK));
        ReflectionTestUtils.setField(flightMapper, "aircraftRepository", aircraftRepositoryMock);
    }

    @Test
    public void dtoToEntity() {
        FlightDto dto = new FlightDto(22L, SRC_AIRPORT_ID, DST_AIRPORT_ID, AIRCRAFT_ID, "ICAO",
                DEPARTURE_TIME, ARRIVAL_TIME, CHECKOUT_START, CHECKOUT_END);

        Flight entity = flightMapper.dtoToEntity(dto);

        assertNull(entity.getId());
        assertEquals(DST_AIRPORT_MOCK, entity.getDestinationAirport());
        assertEquals(SRC_AIRPORT_MOCK, entity.getSourceAirport());
        assertEquals(AIRCRAFT_MOCK, entity.getAircraft());
        assertEquals(CHECKOUT_START, entity.getCheckoutStart());
        assertEquals(CHECKOUT_END, entity.getCheckoutEnd());
        assertEquals(DEPARTURE_TIME, entity.getDepartureTime());
        assertEquals(ARRIVAL_TIME, entity.getArrivalTime());
        assertEquals("ICAO", entity.getAirlaneIcaoCode());
    }

    @Test
    public void toDto() {
        Flight entity = new Flight(22L, SRC_AIRPORT_MOCK, DST_AIRPORT_MOCK, AIRCRAFT_MOCK, "ICAO",
                DEPARTURE_TIME, ARRIVAL_TIME, CHECKOUT_START, CHECKOUT_END, null);

        FlightDto dto = flightMapper.toDto(entity);

        assertEquals(22L, dto.getId());
        assertEquals(SRC_AIRPORT_ID, dto.getSourceAirportId());
        assertEquals(DST_AIRPORT_ID, dto.getDestinationAirportId());
        assertEquals(AIRCRAFT_ID, dto.getAircraftId());
        assertEquals(CHECKOUT_START, dto.getCheckoutStart());
        assertEquals(CHECKOUT_END, dto.getCheckoutEnd());
        assertEquals(DEPARTURE_TIME, dto.getDepartureTime());
        assertEquals(ARRIVAL_TIME, dto.getArrivalTime());
        assertEquals("ICAO", dto.getAirlaneIcaoCode());
    }

    @Test
    public void copyParametersFromDto() {
        Flight entity = new Flight(1337L, mock(Airport.class), mock(Airport.class), mock(Aircraft.class), "ICAOOOO",
                Instant.EPOCH, Instant.EPOCH, Instant.EPOCH, Instant.EPOCH, null);
        FlightDto dto = new FlightDto(22L, SRC_AIRPORT_ID, DST_AIRPORT_ID, AIRCRAFT_ID, "ICAO",
                DEPARTURE_TIME, ARRIVAL_TIME, CHECKOUT_START, CHECKOUT_END);

        flightMapper.copyParametersFromDto(dto, entity);

        assertEquals(1337L, entity.getId());
        assertEquals(DST_AIRPORT_MOCK, entity.getDestinationAirport());
        assertEquals(SRC_AIRPORT_MOCK, entity.getSourceAirport());
        assertEquals(AIRCRAFT_MOCK, entity.getAircraft());
        assertEquals(CHECKOUT_START, entity.getCheckoutStart());
        assertEquals(CHECKOUT_END, entity.getCheckoutEnd());
        assertEquals(DEPARTURE_TIME, entity.getDepartureTime());
        assertEquals(ARRIVAL_TIME, entity.getArrivalTime());
        assertEquals("ICAO", entity.getAirlaneIcaoCode());
    }

}