package lupenko.airport.web.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import lupenko.airport.dao.AircraftRepository;
import lupenko.airport.domain.Aircraft;
import lupenko.airport.web.dto.AircraftDto;
import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@AutoConfigureMockMvc
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class AircraftControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private AircraftRepository aircraftRepository;

    @Autowired
    private ObjectMapper objectMapper;

    @AfterEach
    public void cleanup() {
        aircraftRepository.deleteAll();
    }

    @Test
    public void getAllAircraftDefaultPaging_oneAircraftInDb_aircraftReturned() throws Exception {
        Aircraft aircraft = new Aircraft(10L, "TEST");
        aircraftRepository.save(aircraft);

        mockMvc.perform(MockMvcRequestBuilders.get("/aircraft")
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.content[0].id", is(aircraft.getId())))
                .andExpect(jsonPath("$.content[0].code", Matchers.is(aircraft.getCode())))
                .andExpect(jsonPath("$.content[0].capacity", is(aircraft.getCapacity())))
                .andExpect(jsonPath("$.totalElements", Matchers.is(1)))
        ;
    }

    @Test
    public void createAircraft_emptyDb_aircraftReturnedAndDbContains() throws Exception {
        AircraftDto aircraftDto = new AircraftDto(0L, 10L, "TEST");

        mockMvc.perform(MockMvcRequestBuilders.post("/aircraft")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(aircraftDto)))
                .andDo(print())
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.code", Matchers.is(aircraftDto.getCode())))
                .andExpect(jsonPath("$.capacity", is(aircraftDto.getCapacity())))
        ;

        assertEquals(1, aircraftRepository.count());
    }

    @Test
    public void deleteAircraft_aircraftInDb_statusNoContentAndAircraftDeleted() throws Exception {
        Aircraft aircraft = new Aircraft(10L, "TEST");
        aircraftRepository.save(aircraft);

        mockMvc.perform(MockMvcRequestBuilders.delete("/aircraft/{id}", aircraft.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        assertEquals(0, aircraftRepository.count());
    }

    @Test
    public void getAircraft_aircraftInDb_aircraftReturned() throws Exception {
        Aircraft aircraft = new Aircraft(10L, "TEST");
        aircraftRepository.save(aircraft);

        mockMvc.perform(MockMvcRequestBuilders.get("/aircraft/{id}", aircraft.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(aircraft.getId())))
                .andExpect(jsonPath("$.code", Matchers.is(aircraft.getCode())))
                .andExpect(jsonPath("$.capacity", is(aircraft.getCapacity())))
        ;
    }

    @Test
    public void updateAircraft_aircraftInDb_noContentReturnedAndDbUpdated() throws Exception {
        Aircraft aircraft = new Aircraft(10L, "TEST");
        aircraftRepository.save(aircraft);

        AircraftDto aircraftDto = new AircraftDto(0L, 15L, "TEST_UPDATED");

        mockMvc.perform(MockMvcRequestBuilders.put("/aircraft/{id}", aircraft.getId())
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(aircraftDto)))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        Aircraft updatedAircraft = aircraftRepository.findById(aircraft.getId()).get();
        assertEquals(aircraftDto.getCapacity(), updatedAircraft.getCapacity());
        assertEquals(aircraftDto.getCode(), updatedAircraft.getCode());
    }

    public static Matcher<Integer> is(Long value) {
        return Matchers.is(value.intValue());
    }
}
