package lupenko.airport.web.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import lupenko.airport.dao.AirportRepository;
import lupenko.airport.domain.Airport;
import lupenko.airport.web.dto.AirportDto;
import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import java.math.BigDecimal;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@AutoConfigureMockMvc
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class AirportControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private AirportRepository airportRepository;

    @Autowired
    private ObjectMapper objectMapper;

    @AfterEach
    public void cleanup() {
        airportRepository.deleteAll();
    }

    @Test
    public void getAllAirportDefaultPaging_oneAirportInDb_airportReturned() throws Exception {
        Airport airport = new Airport("TST", "CZ", new BigDecimal(12), new BigDecimal(12));
        airportRepository.save(airport);

        mockMvc.perform(MockMvcRequestBuilders.get("/airports")
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.content[0].id", is(airport.getId())))
                .andExpect(jsonPath("$.content[0].code", Matchers.is(airport.getCode())))
                .andExpect(jsonPath("$.content[0].countryCode", Matchers.is(airport.getCountryCode())))
                .andExpect(jsonPath("$.totalElements", Matchers.is(1)))
        ;
    }

    @Test
    public void createAirport_emptyDb_airportReturnedAndDbContains() throws Exception {
        AirportDto airportDto = new AirportDto(0L, "TST", "CZ", new BigDecimal(12), new BigDecimal(12));

        MvcResult res = mockMvc.perform(MockMvcRequestBuilders.post("/airports")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(airportDto)))
                .andDo(print())
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.code", Matchers.is(airportDto.getCode())))
                .andExpect(jsonPath("$.countryCode", Matchers.is(airportDto.getCountryCode())))
                .andReturn();
        Airport airport = objectMapper.readValue(res.getResponse().getContentAsString(), Airport.class);
        assertThat(airportDto.getDdLatitude(), Matchers.comparesEqualTo(airport.getDdLatitude()));
        assertThat(airportDto.getDdLongitude(), Matchers.comparesEqualTo(airport.getDdLongitude()));

        assertEquals(1, airportRepository.count());
    }

    @Test
    public void deleteAirport_airportInDb_statusNoContentAndDbEmpty() throws Exception {
        Airport airport = new Airport("TST", "CZ", new BigDecimal(12), new BigDecimal(12));
        airportRepository.save(airport);

        mockMvc.perform(MockMvcRequestBuilders.delete("/airports/{id}", airport.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        assertEquals(0, airportRepository.count());
    }

    @Test
    public void getAirport_airportInDb_airportReturned() throws Exception {
        Airport airport = new Airport("TST", "CZ", new BigDecimal(12), new BigDecimal(12));
        airportRepository.save(airport);

        MvcResult res = mockMvc.perform(MockMvcRequestBuilders.get("/airports/{id}", airport.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(airport.getId())))
                .andExpect(jsonPath("$.code", Matchers.is(airport.getCode())))
                .andExpect(jsonPath("$.countryCode", Matchers.is(airport.getCountryCode())))
                .andReturn();

        Airport airportRes = objectMapper.readValue(res.getResponse().getContentAsString(), Airport.class);
        assertThat(airportRes.getDdLatitude(), Matchers.comparesEqualTo(airport.getDdLatitude()));
        assertThat(airportRes.getDdLongitude(), Matchers.comparesEqualTo(airport.getDdLongitude()));
    }

    @Test
    public void updateAirport_airportInDb_noContentReturnedAndDbUpdated() throws Exception {
        Airport airport = new Airport("TST", "CZ", new BigDecimal(12), new BigDecimal(12));
        airportRepository.save(airport);

        AirportDto airportDto = new AirportDto(0L, "TSTUS", "US", new BigDecimal(123), new BigDecimal(123));

        mockMvc.perform(MockMvcRequestBuilders.put("/airports/{id}", airport.getId())
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(airportDto)))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        Airport updatedAirport = airportRepository.findById(airport.getId()).get();
        assertEquals(airportDto.getCode(), updatedAirport.getCode());
        assertThat(airportDto.getDdLatitude(), Matchers.comparesEqualTo(updatedAirport.getDdLatitude()));
        assertThat(airportDto.getDdLongitude(), Matchers.comparesEqualTo(updatedAirport.getDdLongitude()));
        assertEquals(airportDto.getCountryCode(), updatedAirport.getCountryCode());

    }

    public static Matcher<Integer> is(Long value) {
        return Matchers.is(value.intValue());
    }
}
