package lupenko.airport.web.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import lupenko.airport.dao.*;
import lupenko.airport.domain.*;
import lupenko.airport.web.dto.BookingDto;
import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@AutoConfigureMockMvc
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class BookingControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private BookingRepository bookingRepository;
    @Autowired
    private PassengerRepository passengerRepository;
    @Autowired
    private AircraftRepository aircraftRepository;
    @Autowired
    private AirportRepository airportRepository;
    @Autowired
    private FlightRepository flightRepository;

    @Autowired
    private ObjectMapper objectMapper;

    @AfterEach
    public void cleanup() {
        bookingRepository.deleteAll();
        passengerRepository.deleteAll();
        flightRepository.deleteAll();
        aircraftRepository.deleteAll();
        airportRepository.deleteAll();
    }

    @Test
    public void getAllBookingDefaultPaging_oneBookingInDb_bookingReturned() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        Booking booking = new Booking(flight, passenger);
        booking = bookingRepository.save(booking);

        mockMvc.perform(MockMvcRequestBuilders.get("/bookings")
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.content[0].id", is(booking.getId())))
                .andExpect(jsonPath("$.content[0].flightId", is(booking.getFlight().getId())))
                .andExpect(jsonPath("$.content[0].passengerId", is(booking.getPassenger().getId())))
                .andExpect(jsonPath("$.content[0].checkout").doesNotExist())
                .andExpect(jsonPath("$.totalElements", Matchers.is(1)))
        ;
    }

    @Test
    public void createBooking_emptyDb_bookingReturnedAndDbContains() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        flightRepository.save(flight);
        passengerRepository.save(passenger);

        BookingDto bookingDto = new BookingDto(flight.getId(), passenger.getId());

        mockMvc.perform(MockMvcRequestBuilders.post("/bookings")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(bookingDto)))
                .andDo(print())
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.flightId", is(bookingDto.getFlightId())))
                .andExpect(jsonPath("$.passengerId", is(bookingDto.getPassengerId())))
                .andExpect(jsonPath("$.checkout").doesNotExist())
        ;

        assertEquals(1, bookingRepository.count());
    }

    @Test
    public void deleteBooking_bookingInDb_statusNoContentAndBookingDeleted() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        Booking booking = new Booking(flight, passenger);
        booking = bookingRepository.save(booking);

        mockMvc.perform(MockMvcRequestBuilders.delete("/bookings/{id}", booking.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        assertEquals(0, bookingRepository.count());
    }

    @Test
    public void getBooking_bookingInDb_bookingReturned() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        Booking booking = new Booking(flight, passenger);
        booking = bookingRepository.save(booking);

        mockMvc.perform(MockMvcRequestBuilders.get("/bookings/{id}", booking.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(booking.getId())))
                .andExpect(jsonPath("$.flightId", is(booking.getFlight().getId())))
                .andExpect(jsonPath("$.passengerId", is(booking.getPassenger().getId())))
                .andExpect(jsonPath("$.checkout").doesNotExist())
        ;
    }

    @Test
    public void setBookingCheckoutTime_bookingInDb_noContentReturnedAndDbUpdated() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        Booking booking = new Booking(flight, passenger);
        booking = bookingRepository.save(booking);

        String checkoutTime = Instant.now().toString();
        mockMvc.perform(MockMvcRequestBuilders.patch("/bookings/{id}", booking.getId())
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .param("checkoutTime", checkoutTime))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        Booking updatedBooking = bookingRepository.findById(booking.getId()).get();
        assertEquals(Instant.parse(checkoutTime).truncatedTo(ChronoUnit.SECONDS), updatedBooking.getCheckout().truncatedTo(ChronoUnit.SECONDS));
    }

    public static Matcher<Integer> is(Long value) {
        return Matchers.is(value.intValue());
    }
}
