package lupenko.airport.web.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import lupenko.airport.dao.AircraftRepository;
import lupenko.airport.dao.AirportRepository;
import lupenko.airport.dao.FlightRepository;
import lupenko.airport.domain.Aircraft;
import lupenko.airport.domain.Airport;
import lupenko.airport.domain.Flight;
import lupenko.airport.web.dto.FlightDto;
import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.temporal.ChronoUnit;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@AutoConfigureMockMvc
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class FlightControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private FlightRepository flightRepository;
    @Autowired
    private AircraftRepository aircraftRepository;
    @Autowired
    private AirportRepository airportRepository;

    @Autowired
    private ObjectMapper objectMapper;

    @AfterEach
    public void cleanup() {
        flightRepository.deleteAll();
        aircraftRepository.deleteAll();
        airportRepository.deleteAll();
    }

    @Test
    public void getAllFlightDefaultPaging_oneFlightInDb_flightReturned() throws Exception {
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        flightRepository.save(flight);

        mockMvc.perform(MockMvcRequestBuilders.get("/flights")
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.content[0].id", is(flight.getId())))
                .andExpect(jsonPath("$.content[0].airlaneIcaoCode", Matchers.is(flight.getAirlaneIcaoCode())))
                .andExpect(jsonPath("$.content[0].destinationAirportId", is(flight.getDestinationAirport().getId())))
                .andExpect(jsonPath("$.content[0].sourceAirportId", is(flight.getSourceAirport().getId())))
                .andExpect(jsonPath("$.content[0].aircraftId", is(flight.getAircraft().getId())))
                .andExpect(jsonPath("$.totalElements", Matchers.is(1)))
        ;
    }

    @Test
    public void createFlight_emptyDb_flightReturnedAndDbContains() throws Exception {
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        aircraftRepository.save(aircraft);
        airportRepository.save(airport);

        FlightDto flightDto = new FlightDto(airport.getId(), airport.getId(), aircraft.getId(), "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());

        mockMvc.perform(MockMvcRequestBuilders.post("/flights")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(flightDto)))
                .andDo(print())
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.airlaneIcaoCode", Matchers.is(flightDto.getAirlaneIcaoCode())))
                .andExpect(jsonPath("$.destinationAirportId", is(flightDto.getDestinationAirportId())))
                .andExpect(jsonPath("$.sourceAirportId", is(flightDto.getSourceAirportId())))
                .andExpect(jsonPath("$.aircraftId", is(flightDto.getAircraftId())))
        ;

        assertEquals(1, flightRepository.count());
    }

    @Test
    public void deleteFlight_flightInDb_statusNoContentAndFlightDeleted() throws Exception {
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        flightRepository.save(flight);

        mockMvc.perform(MockMvcRequestBuilders.delete("/flights/{id}", flight.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        assertEquals(0, flightRepository.count());
    }

    @Test
    public void getFlight_flightInDb_flightReturned() throws Exception {
        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        flightRepository.save(flight);

        mockMvc.perform(MockMvcRequestBuilders.get("/flights/{id}", flight.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(flight.getId())))
                .andExpect(jsonPath("$.airlaneIcaoCode", Matchers.is(flight.getAirlaneIcaoCode())))
                .andExpect(jsonPath("$.destinationAirportId", is(flight.getDestinationAirport().getId())))
                .andExpect(jsonPath("$.sourceAirportId", is(flight.getSourceAirport().getId())))
                .andExpect(jsonPath("$.aircraftId", is(flight.getAircraft().getId())))
        ;
    }

    @Test
    public void updateFlight_flightInDb_noContentReturnedAndDbUpdated() throws Exception {
        Airport airport1 = new Airport("TEST1", "CZ", new BigDecimal(212), new BigDecimal(212));
        Aircraft aircraft1 = new Aircraft(1001L, "A11");
        airportRepository.save(airport1);
        aircraftRepository.save(aircraft1);

        Airport airport = new Airport("TEST", "CZ", new BigDecimal(12), new BigDecimal(12));
        Aircraft aircraft = new Aircraft(100L, "A1");
        Flight flight = new Flight(airport, airport, aircraft, "CSA", Instant.now(), Instant.now(), Instant.now(), Instant.now());
        flightRepository.save(flight);

        FlightDto flightDto = new FlightDto(airport1.getId(), airport1.getId(), aircraft1.getId(), "SUP", Instant.now(), Instant.now(), Instant.now(), Instant.now());

        mockMvc.perform(MockMvcRequestBuilders.put("/flights/{id}", flight.getId())
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(flightDto)))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        Flight updatedFlight = flightRepository.findById(flight.getId()).get();
        assertEquals(flightDto.getDestinationAirportId(), updatedFlight.getDestinationAirport().getId());
        assertEquals(flightDto.getSourceAirportId(), updatedFlight.getSourceAirport().getId());
        assertEquals(flightDto.getAircraftId(), updatedFlight.getAircraft().getId());
        assertEquals(flightDto.getAirlaneIcaoCode(), updatedFlight.getAirlaneIcaoCode());
        assertEquals(flightDto.getArrivalTime().truncatedTo(ChronoUnit.SECONDS), updatedFlight.getArrivalTime().truncatedTo(ChronoUnit.SECONDS));
        assertEquals(flightDto.getDepartureTime().truncatedTo(ChronoUnit.SECONDS), updatedFlight.getDepartureTime().truncatedTo(ChronoUnit.SECONDS));
        assertEquals(flightDto.getCheckoutEnd().truncatedTo(ChronoUnit.SECONDS), updatedFlight.getCheckoutEnd().truncatedTo(ChronoUnit.SECONDS));
        assertEquals(flightDto.getCheckoutStart().truncatedTo(ChronoUnit.SECONDS), updatedFlight.getCheckoutStart().truncatedTo(ChronoUnit.SECONDS));
    }

    public static Matcher<Integer> is(Long value) {
        return Matchers.is(value.intValue());
    }
}
