package lupenko.airport.web.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import lupenko.airport.dao.PassengerRepository;
import lupenko.airport.domain.Passenger;
import lupenko.airport.web.dto.PassengerDto;
import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import java.time.LocalDate;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@AutoConfigureMockMvc
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class PassengerControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private PassengerRepository passengerRepository;

    @Autowired
    private ObjectMapper objectMapper;

    @AfterEach
    public void cleanup() {
        passengerRepository.deleteAll();
    }

    @Test
    public void getAllPassengerDefaultPaging_onePassengerInDb_passengerReturned() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        passengerRepository.save(passenger);

        mockMvc.perform(MockMvcRequestBuilders.get("/passengers")
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.content[0].id", is(passenger.getId())))
                .andExpect(jsonPath("$.content[0].address", Matchers.is(passenger.getAddress())))
                .andExpect(jsonPath("$.content[0].birthDate", Matchers.is(passenger.getBirthDate().toString())))
                .andExpect(jsonPath("$.content[0].firstName", Matchers.is(passenger.getFirstName())))
                .andExpect(jsonPath("$.content[0].lastName", Matchers.is(passenger.getLastName())))
                .andExpect(jsonPath("$.content[0].passport", Matchers.is(passenger.getPassport())))
                .andExpect(jsonPath("$.totalElements", Matchers.is(1)))
        ;
    }

    @Test
    public void createPassenger_emptyDb_passengerReturnedAndDbContains() throws Exception {
        PassengerDto passengerDto = new PassengerDto("Name", "LastName", "CZ", LocalDate.now(), "ASD123");

        mockMvc.perform(MockMvcRequestBuilders.post("/passengers")
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(passengerDto)))
                .andDo(print())
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.address", Matchers.is(passengerDto.getAddress())))
                .andExpect(jsonPath("$.birthDate", Matchers.is(passengerDto.getBirthDate().toString())))
                .andExpect(jsonPath("$.firstName", Matchers.is(passengerDto.getFirstName())))
                .andExpect(jsonPath("$.lastName", Matchers.is(passengerDto.getLastName())))
                .andExpect(jsonPath("$.passport", Matchers.is(passengerDto.getPassport())))
        ;

        assertEquals(1, passengerRepository.count());
    }

    @Test
    public void deletePassenger_passengerInDb_statusNoContentAndPassengerDeleted() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        passengerRepository.save(passenger);

        mockMvc.perform(MockMvcRequestBuilders.delete("/passengers/{id}", passenger.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        assertEquals(0, passengerRepository.count());
    }

    @Test
    public void getPassenger_passengerInDb_passengerReturned() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        passengerRepository.save(passenger);

        mockMvc.perform(MockMvcRequestBuilders.get("/passengers/{id}", passenger.getId())
                .accept(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(passenger.getId())))
                .andExpect(jsonPath("$.address", Matchers.is(passenger.getAddress())))
                .andExpect(jsonPath("$.birthDate", Matchers.is(passenger.getBirthDate().toString())))
                .andExpect(jsonPath("$.firstName", Matchers.is(passenger.getFirstName())))
                .andExpect(jsonPath("$.lastName", Matchers.is(passenger.getLastName())))
                .andExpect(jsonPath("$.passport", Matchers.is(passenger.getPassport())))
        ;
    }

    @Test
    public void updatePassenger_passengerInDb_noContentReturnedAndDbUpdated() throws Exception {
        Passenger passenger = new Passenger("Name", "LastName", "CZ", LocalDate.now(), "ASD123");
        passengerRepository.save(passenger);

        PassengerDto passengerDto = new PassengerDto("Name1", "LastName1", "US", LocalDate.now(), "ASD1235");

        mockMvc.perform(MockMvcRequestBuilders.put("/passengers/{id}", passenger.getId())
                .contentType(MediaType.APPLICATION_JSON)
                .accept(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(passengerDto)))
                .andDo(print())
                .andExpect(status().isNoContent())
        ;

        Passenger updatedPassenger = passengerRepository.findById(passenger.getId()).get();
        assertEquals(passengerDto.getAddress(), updatedPassenger.getAddress());
        assertEquals(passengerDto.getBirthDate(), updatedPassenger.getBirthDate());
        assertEquals(passengerDto.getFirstName(), updatedPassenger.getFirstName());
        assertEquals(passengerDto.getLastName(), updatedPassenger.getLastName());
        assertEquals(passengerDto.getPassport(), updatedPassenger.getPassport());
    }

    public static Matcher<Integer> is(Long value) {
        return Matchers.is(value.intValue());
    }
}
